from math import atan, degrees

BALL_FRICTION = .1
# BALL_MASS = 0.013  # [kg]
BALL_MASS = 0.006  # [kg]
# BALL_RADIUS = 0.0105  # [m]
BALL_RADIUS = 0.008  # [m]
BALL_RESTITUTION = 0.7
BLOCKER_LWH = (.01, .025, .005)  # [m]
BOX_LWHT = (.133, .081, .087, .003)  # [m]
BOX_MASS = .027  # [kg]
COIN_HEIGHT = .005  # [m]
COIN_MASS = .011  # [kg]
COIN_RADIUS = .020  # [m]
DOMINO_LWH = [.007, .02, .044]
DOMINO_MASS = .00305
DOMINO_FRICTION = 0.5
DOMINO_RESTITUTION = 0.7
DOMINO_SUPPORT_LWH = [.1, .025, .005]  # [m]
FLAT_SUPPORT_LWH = (.02, .025, .005)  # [m]
GOBLET_ANGULAR_DAMPING = 1
GOBLET_LINEAR_DAMPING = .1
GOBLET_FRICTION = 1
GOBLET_HEIGHT = 0.11  # [m]
GOBLET_MASS = .005  # [g]
GOBLET_R1 = 0.036  # [m]
GOBLET_R2 = 0.025  # [m]
GOBLET_EPS = .004  # [m]
COIN_GOBLET_HEIGHT = 0.082  # [m]
COIN_GOBLET_MASS = .04  # [g]
COIN_GOBLET_R1 = 0.0415  # [m]
COIN_GOBLET_R2 = 0.0325  # [m]
LEFT_PULLEY_ROPE_LENGTH = .95  # [m]
LEVER_ANGULAR_DAMPING = .8
LONG_LEVER_LWH = [.35, .025, .01]  # [m]
LONG_TRACK_LWHT = (0.3, 0.025, 0.008, .002)  # [m]
LONG_TRACK_MASS = .01  # [kg] TODO
MEDIUM_WEIGHT_HEIGHT = 0.02  # [m]
MEDIUM_WEIGHT_RADIUS = 0.01  # [m]
MEDIUM_WEIGHT_MASS = 0.05  # [kg]
NAIL_LEVER_LWH = (0.11, 0.005, 0.002)  # [m]
NAIL_LEVER_MASS = 0.01  # [kg]
PIVOT_LENGTH = .06  # [m]
PIVOT_RADIUS = .004  # [m]
PLANK_LWH = (0.1175, 0.023, 0.008)  # [m]
PLANK_MASS = 0.01  # [kg]
PLANK_RESTITUTION = 0.8
QUAD_PLANK_LWH = (0.031, 0.023, 0.1175)  # [m]
RIGHT_PULLEY_PIVOT_HEIGHT = .003  # [m]
RIGHT_PULLEY_PIVOT_RADIUS = .006  # [m]
RIGHT_PULLEY_ROPE_LENGTH = 1.  # [m]
RIGHT_WEIGHT_HEIGHT = 0.0315  # [m]
RIGHT_WEIGHT_MASS = 0.2  # [kg]
RIGHT_WEIGHT_RADIUS = 0.0315 / 2  # [m]
SHORT_TRACK_LWHT = (0.15, 0.025, 0.008, .002)  # [m]
SMOL_TRACK_LWHT = (0.1, 0.025, 0.008, .002)  # [m]
SMOL_WEIGHT_HEIGHT = 0.014  # [m]
SMOL_WEIGHT_RADIUS = 0.0075  # [m]
SMOL_WEIGHT_MASS = 0.02  # [kg]

ANGVEL = 10
PIVOTING_ANGULAR_VELOCITY = 1
PLANK_TOPPLING_ANGLE = degrees(atan(PLANK_LWH[2] / PLANK_LWH[0]))
FALLING_LINEAR_VELOCITY = .01

DATA = {
    'scene': [
        # ------------------------- BOX-LEVER GROUP --------------------------
        {
            'name': "box_lever_origin",
            'type': "Empty",
            'args': {},
            'xform': {
                'value': [.25, 0, .10, 0, 0, 0],
                'range': [
                    [.23, .27],
                    None,
                    [.08, .12],
                    None,
                    None,
                    None
                ]
            }
        },
        {
            'name': "box_lever_plank",
            'type': "Box",
            'args': {
                'extents': [PLANK_LWH[0]*2, PLANK_LWH[1], PLANK_LWH[2]],
                'b_mass': 2*PLANK_MASS,
            },
            'parent': "box_lever_origin",
            'xform': {
                'value': [0, 0, 0, 0, 0, 0],
            }
        },
        {
            'name': "box_lever_pivot",
            'type': "Pivot",
            'args': {
                'pivot_pos': [0, 0, -PIVOT_RADIUS],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
            },
            # 'parent': "box_lever_origin",
            'components': ["box_lever_plank"],
        },
        {
            'name': "box_lever_counterweight",
            'type': "Cylinder",
            'args': {
                'extents': [SMOL_WEIGHT_RADIUS, SMOL_WEIGHT_HEIGHT],
                'b_mass': SMOL_WEIGHT_MASS
            },
            'parent': "box_lever_plank",
            'xform': {
                'value': [-PLANK_LWH[0]+SMOL_WEIGHT_RADIUS,
                          0,
                          SMOL_WEIGHT_HEIGHT/2+PLANK_LWH[2]/2,
                          0, 0, 0],
            }
        },
        {
            'name': "box_lever_counterweight_fastener",
            'type': "Fastener",
            'args': {
                'comp1_xform': [0, 0, -SMOL_WEIGHT_HEIGHT/2, 0, 0, 0],
                'comp2_xform': [-PLANK_LWH[0]+SMOL_WEIGHT_RADIUS,
                                0,
                                PLANK_LWH[2]/2,
                                0, 0, 0]
            },
            'components': ["box_lever_counterweight",
                           "box_lever_plank"]
        },
        {
            'name': "box_lever_blocker_left",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'parent': "box_lever_origin",
            'xform': {
                'value': [-PLANK_LWH[0], 0, -PLANK_LWH[2], 90, 0, 0],
            }
        },
        {
            'name': "box_lever_blocker_right",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'parent': "box_lever_origin",
            'xform': {
                'value': [PLANK_LWH[0], 0, -.02, 90, 0, 0],
            }
        },
        {
            'name': "box",
            'type': "OpenBox",
            'args': {
                'extents': BOX_LWHT,
                'b_mass': BOX_MASS,
            },
            'parent': "box_lever_plank",
            'xform': {
                'value': [PLANK_LWH[0]-BOX_LWHT[0]/2,
                          0,
                          PLANK_LWH[2]/2+BOX_LWHT[2]/2,
                          0, 0, 0],
            }
        },
        {
            'name': "box_fastener",
            'type': "Fastener",
            'args': {
                'comp1_xform': [0, 0, -BOX_LWHT[2]/2-BOX_LWHT[3]/2, 0, 0, 0],
                'comp2_xform': [PLANK_LWH[0]-BOX_LWHT[0]/2-.01,
                                0,
                                PLANK_LWH[2]/2,
                                0, 0, 0]
            },
            'components': ["box", "box_lever_plank"]
        },
        {
            'name': "box_weight",
            'type': "Cylinder",
            'args': {
                'extents': [SMOL_WEIGHT_RADIUS, SMOL_WEIGHT_HEIGHT],
                'b_mass': SMOL_WEIGHT_MASS
            },
            'parent': "box",
            'xform': {
                'value': [BOX_LWHT[0]/3, 0, -BOX_LWHT[2]/3, 0, 0, 0],
            }
        },
        {
            'name': "box_rope",
            'type': "TensionRope",
            'args': {
                'comp1_pos': [BOX_LWHT[0]/2, 0, BOX_LWHT[2]/2],
                'comp2_pos': [PLANK_LWH[0]/2, 0, 0],
                'pivot_hpr': [0, 90, 0],
                'hook_radius': .005,
                'loose_rope': .01
            },
            'components': ["box", "initial_run_lever_plank"]
        },
        # ------------------------ DOMINO-BALL GROUP -------------------------
        {
            'name': "initial_run_lever_plank",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH,
                'b_mass': PLANK_MASS,
                'b_angular_damping': LEVER_ANGULAR_DAMPING
            },
            'xform': {
                'value': [.2+PLANK_LWH[0]-.009, 0, .58, 0, 0, 0],
            }
        },
        {
            'name': "initial_run_lever_pusher",
            'type': "Box",
            'args': {
                'extents': DOMINO_LWH,
                'b_mass': DOMINO_MASS,
            },
            'parent': "initial_run_lever_plank",
            'xform': {
                'value': [-PLANK_LWH[0]/2+DOMINO_LWH[0]/2,
                          0,
                          -PLANK_LWH[2]/2-DOMINO_LWH[2]/2,
                          0, 0, 0],
            }
        },
        {
            'name': "fastener2",
            'type': "Fastener",
            'args': {
                'comp1_xform': [
                    -PLANK_LWH[0]/2+DOMINO_LWH[0]/2, 0, -PLANK_LWH[2]/2,
                    0, 0, 0
                ],
                'comp2_xform': [0, 0, DOMINO_LWH[2]/2, 0, 0, 0]
            },
            'components': ["initial_run_lever_plank",
                           "initial_run_lever_pusher"]
        },
        {
            'name': "initial_run_lever_pivot",
            'type': "Pivot",
            'args': {
                'pivot_pos': [0, 0, .02],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
            },
            'components': ["initial_run_lever_plank"],
        },
        {
            'name': "initial_run_support",
            'type': "Box",
            'args': {
                'extents': DOMINO_SUPPORT_LWH,
                'b_friction': DOMINO_FRICTION
            },
            'xform': {
                'value': [.2, 0, .5, 0, 0, 0]
            }
        },
        {
            'name': "initial_run",
            'type': "DominoRun",
            'args': {
                'extents': DOMINO_LWH,
                'coords': [[.005, 0, 0], [.025, 0, 0], [.045, 0, 0]],
                'b_mass': DOMINO_MASS,
                'b_friction': DOMINO_FRICTION
            },
            'parent': "initial_run_support",
            'xform': {
                'value': [DOMINO_SUPPORT_LWH[0]/2, 0, DOMINO_SUPPORT_LWH[2]/2,
                          180, 0, 0]
            }
        },
        {
            'name': "ball1",
            'type': "Ball",
            'args': {
                'radius': BALL_RADIUS,
                'b_mass': BALL_MASS,
                'b_restitution': BALL_RESTITUTION
            },
            'parent': "initial_run_support",
            'xform': {
                'value': [-DOMINO_SUPPORT_LWH[0]/2+.002,
                          0,
                          DOMINO_SUPPORT_LWH[2]/2+BALL_RADIUS,
                          0, 0, 0]
            }
        },
        {
            'name': "ball1_track_origin",
            'type': "Empty",
            'args': {},
            'parent': "initial_run_support",
            'xform': {
                'value': [-DOMINO_SUPPORT_LWH[0]/2,
                          0,
                          -DOMINO_SUPPORT_LWH[2]/2,
                          0, 0, -13],
                'range': [
                    None,
                    None,
                    None,
                    None,
                    None,
                    [-15, -11]
                ]
            }
        },
        {
            'name': "ball1_track",
            'type': "Track",
            'args': {
                'extents': LONG_TRACK_LWHT
            },
            'parent': "ball1_track_origin",
            'xform': {
                'value': [-LONG_TRACK_LWHT[0]/2, 0, LONG_TRACK_LWHT[2]/2,
                          0, 0, 0],
            }
        },
        # ------------------------- BOUNCER (ALONE?) -------------------------
        {
            'name': "ball1_bouncer",
            'type': "Box",
            'args': {
                'extents': DOMINO_LWH,
                'b_restitution': DOMINO_RESTITUTION,
            },
            'xform': {
                'value': [-.17, 0, .42, 90, 0, 90],
                'range': [
                    [-.19, -.15],
                    None,
                    [.40, .44],
                    None,
                    None,
                    None
                ]
            }
        },
        # -------------------------- LEVERS GROUP ----------------------------
        {
            'name': "ball1_goblet",
            'type': "Goblet",
            'args': {
                'extents': [
                    GOBLET_HEIGHT,
                    GOBLET_R1,
                    GOBLET_R2,
                    GOBLET_EPS
                ],
            },
            'parent': "falling_plank_support",
            'xform': {
                'value': [0, -.07, .18, 0, -15, 0],
            }
        },
        {
            'name': "lever1",
            'type': "Lever",
            'args': {
                'extents': PLANK_LWH,
                'pivot_pos': [0, 0, .005],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
                'b_mass': PLANK_MASS,
                'b_restitution': DOMINO_RESTITUTION
            },
            'parent': "falling_plank_support",
            'xform': {
                'value': [0, PLANK_LWH[1]/3, PLANK_LWH[0]*2.45, 90, 0, 90],
            }
        },
        {
            'name': "lever1_blocker",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'parent': "falling_plank_support",
            'xform': {
                'value': [0, .024, PLANK_LWH[0]*2.2, 0, 0, 0],
            }
        },
        {
            'name': "lever2",
            'type': "Lever",
            'args': {
                'extents': PLANK_LWH,
                'pivot_pos': [0, 0, -.005],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
                'b_mass': PLANK_MASS
            },
            'parent': "falling_plank_support",
            'xform': {
                'value': [0, -.001, PLANK_LWH[0]*1.5, 90, 0, 90],
            }
        },
        {
            'name': "falling_plank",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH,
                'b_mass': PLANK_MASS,
            },
            'parent': "falling_plank_support",
            'xform': {
                'value': [0, PLANK_LWH[1]/3, PLANK_LWH[0]/2+PLANK_LWH[2]/2,
                          90, 0, 90],
            }
        },
        {
            'name': "falling_plank_support",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'xform': {
                'value': [-.23, 0, .10, 90, 0, 0],
                'range': [
                    [-.25, -.21],
                    None,
                    [.08, .12],
                    None,
                    None,
                    None
                ]
            }
        },
        # --------------------------- BOTTOM GROUP ---------------------------
        {
            'name': "right_obstacle",
            'type': "Box",
            'args': {
                'extents': [PLANK_LWH[0], PLANK_LWH[1]*2, PLANK_LWH[2]]
            },
            'xform': {
                'value': [-.30, 0, -.05, 90, 0, 0],
                'range': [
                    [-.32, -.28],
                    None,
                    [-.07, -.03],
                    None,
                    None,
                    None
                ]
            }
        },
        {
            'name': "left_obstacle",
            'type': "Box",
            'args': {
                'extents': [PLANK_LWH[0], PLANK_LWH[1]*2, PLANK_LWH[2]]
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, PLANK_LWH[1]*2+.03, 0, 0, 0, 0],
            }
        },
        {
            'name': "lever3_plank",
            'type': "Box",
            'args': {
                'extents': LONG_LEVER_LWH,
                'b_mass': 3*PLANK_MASS
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, -LONG_LEVER_LWH[0]/2+.05, -.15, 90, 0, 0],
            }
        },
        {
            'name': "lever3_pivot",
            'type': "Pivot",
            'args': {
                'pivot_pos': [0, 0, -.008],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
            },
            # 'parent': "right_obstacle",
            'components': ["lever3_plank"],
        },
        {
            'name': "lever3_blocker_right",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH,
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, -LONG_LEVER_LWH[0]*.85, -.16, 0, 0, 0],
            }
        },
        {
            'name': "lever3_blocker_left",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH,
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, 0, -.20, 0, 0, 0],
            }
        },
        {
            'name': "lever3_goblet",
            'type': "Goblet",
            'args': {
                'extents': [
                    GOBLET_HEIGHT,
                    GOBLET_R1,
                    GOBLET_R2,
                    GOBLET_EPS
                ],
                'b_mass': GOBLET_MASS
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, .04, -.15+LONG_LEVER_LWH[2]/2, 0, 0, 0],
            }
        },
        {
            'name': "fastener1",
            'type': "Fastener",
            'args': {
                'comp1_xform': [
                    LONG_LEVER_LWH[0]/2-.01, 0, LONG_LEVER_LWH[2]/2,
                    0, 0, 0
                ],
                'comp2_xform': [0, 0, 0, 0, 0, 0]
            },
            'components': ["lever3_plank", "lever3_goblet"]
        },
        {
            'name': "lever4_plank",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH,
                'b_mass': PLANK_MASS,
                'b_angular_damping': LEVER_ANGULAR_DAMPING
            },
            'parent': "right_obstacle",
            'xform': {
                'value': [0, -LONG_LEVER_LWH[0]*0.95, -.14, 90, 0, 0],
            }
        },
        {
            'name': "lever4_pivot",
            'type': "Pivot",
            'args': {
                'pivot_pos': [-PLANK_LWH[0]/2+PIVOT_RADIUS, 0, -.02],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
            },
            # 'parent': "right_obstacle",
            'components': ["lever4_plank"],
        },
        {
            'name': "lever4_weight",
            'type': "Cylinder",
            'args': {
                'extents': [MEDIUM_WEIGHT_RADIUS, MEDIUM_WEIGHT_HEIGHT],
                'b_mass': MEDIUM_WEIGHT_MASS,
            },
            'parent': "lever4_plank",
            'xform': {
                'value': [-PLANK_LWH[0]/2+.006,
                          0,
                          MEDIUM_WEIGHT_HEIGHT/2+PLANK_LWH[2]/2,
                          180, 0, 0],
            }
        },
        {
            'name': "weight_rope",
            'type': "TensionRope",
            'args': {
                'comp1_pos': [0, 0, MEDIUM_WEIGHT_HEIGHT/2],
                'comp2_pos': [PLANK_LWH[0], 0, -PLANK_LWH[2]/2],
                'pivot_hpr': [0, 90, 0],
                'hook_radius': .005,
                'loose_rope': .05
            },
            'components': ["lever4_weight", "coin_goblet_lever_plank"]
        },
        # ------------------------ COIN GOBLET GROUP -------------------------
        {
            'name': "coin_goblet_origin",
            'type': "Empty",
            'args': {},
            'xform': {
                'value': [.10, 0, .35, 0, 0, 0],
                'range': [
                    [.09, .11],
                    None,
                    None,
                    [.34, .36],
                    None,
                    None,
                    None
                ]
            }
        },
        {
            'name': "coin_goblet",
            'type': "Goblet",
            'args': {
                'extents': [
                    COIN_GOBLET_HEIGHT,
                    COIN_GOBLET_R1,
                    COIN_GOBLET_R2,
                    GOBLET_EPS
                ],
                'b_mass': COIN_GOBLET_MASS
            },
            'parent': "coin_goblet_lever_plank",
            'xform': {
                'value': [PLANK_LWH[0]/3, 0, .04, 0, 0, 84],
            }
        },
        {
            'name': "coin_goblet_fastener",
            'type': "Fastener",
            'args': {
                'comp1_xform': [0, 0, 0, 0, 0, 0],
                'comp2_xform': [PLANK_LWH[0]/3, 0, .04, 0, 0, 84]
            },
            'components': ["coin_goblet", "coin_goblet_lever_plank"]
        },
        {
            'name': "coin_goblet_counterweight",
            'type': "Cylinder",
            'args': {
                'extents': [MEDIUM_WEIGHT_RADIUS, MEDIUM_WEIGHT_HEIGHT],
                'b_mass': MEDIUM_WEIGHT_MASS,
            },
            'parent': "coin_goblet_lever_plank",
            'xform': {
                'value': [-PLANK_LWH[0]+MEDIUM_WEIGHT_RADIUS,
                          0,
                          MEDIUM_WEIGHT_HEIGHT/2+PLANK_LWH[2]/2,
                          0, 0, 0],
            }
        },
        {
            'name': "coin_goblet_counterweight_fastener",
            'type': "Fastener",
            'args': {
                'comp1_xform': [0, 0, -MEDIUM_WEIGHT_HEIGHT/2, 0, 0, 0],
                'comp2_xform': [-PLANK_LWH[0]+MEDIUM_WEIGHT_RADIUS,
                                0,
                                PLANK_LWH[2]/2,
                                0, 0, 0]
            },
            'components': ["coin_goblet_counterweight",
                           "coin_goblet_lever_plank"]
        },
        {
            'name': "coin_goblet_lever_plank",
            'type': "Box",
            'args': {
                'extents': [PLANK_LWH[0]*2, PLANK_LWH[1], PLANK_LWH[2]],
                'b_mass': 2*PLANK_MASS
            },
            'parent': "coin_goblet_origin",
            'xform': {
                'value': [0, 0, 0, 0, 0, -20],
            }
        },
        {
            'name': "coin_goblet_lever_pivot",
            'type': "Pivot",
            'args': {
                'pivot_pos': [0, 0, -.02],
                'pivot_hpr': [0, 90, 0],
                'pivot_extents': [PIVOT_RADIUS, PIVOT_LENGTH],
            },
            # 'parent': "coin_goblet_origin",
            'components': ["coin_goblet_lever_plank"],
        },
        {
            'name': "coin_goblet_lever_blocker_left",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'parent': "coin_goblet_origin",
            'xform': {
                'value': [-PLANK_LWH[0]+.01, 0, -.05, 90, 0, 0],
            }
        },
        {
            'name': "coin_goblet_lever_blocker_right",
            'type': "Box",
            'args': {
                'extents': PLANK_LWH
            },
            'parent': "coin_goblet_origin",
            'xform': {
                'value': [PLANK_LWH[0]/2, 0, -.05, 90, 0, 0],
            }
        },
        {
            'name': "coin1",
            'type': "Cylinder",
            'args': {
                'extents': [COIN_RADIUS, COIN_HEIGHT],
                'b_mass': COIN_MASS
            },
            'parent': "coin_goblet",
            'xform': {
                'value': [0, 0, COIN_HEIGHT*(1+.5)+.002, 0, 0, 0],
            }
        },
        {
            'name': "coin2",
            'type': "Cylinder",
            'args': {
                'extents': [COIN_RADIUS, COIN_HEIGHT],
                'b_mass': COIN_MASS
            },
            'parent': "coin_goblet",
            'xform': {
                'value': [0, 0, COIN_HEIGHT*(2+.5)+.002, 0, 0, 0],
            }
        },
        {
            'name': "coin3",
            'type': "Cylinder",
            'args': {
                'extents': [COIN_RADIUS, COIN_HEIGHT],
                'b_mass': COIN_MASS
            },
            'parent': "coin_goblet",
            'xform': {
                'value': [0, 0, COIN_HEIGHT*(3+.5)+.002, 0, 0, 0],
            }
        },
        {
            'name': "coin4",
            'type': "Cylinder",
            'args': {
                'extents': [COIN_RADIUS, COIN_HEIGHT],
                'b_mass': COIN_MASS
            },
            'parent': "coin_goblet",
            'xform': {
                'value': [0, 0, COIN_HEIGHT*(4+.5)+.002, 0, 0, 0],
            }
        },
    ],
    'causal_graph': [
        {
            'name': "domrun_lever_pivots",
            'type': "Pivoting",
            'args': {
                'body': "initial_run_lever_plank",
                'min_angvel': PIVOTING_ANGULAR_VELOCITY,
            },
            'children': [
                "first_domino_falls_on_second",
            ]
        },
        {
            'name': "first_domino_falls_on_second",
            'type': "Contact",
            'args': {
                'first': "initial_run_dom_0",
                'second': "initial_run_dom_1"
            },
            'children': [
                "last_domino_hits_ball",
            ]
        },
        {
            'name': "last_domino_hits_ball",
            'type': "Contact",
            'args': {
                'first': "initial_run_dom_2",
                'second': "ball1"
            },
            'children': [
                "ball_rolls_on_track",
            ]
        },
        {
            'name': "ball_rolls_on_track",
            'type': "RollingOn",
            'args': {
                'rolling': "ball1",
                'support': "ball1_track",
                'min_angvel': ANGVEL
            },
            'children': [
                "ball_hits_bouncer",
            ]
        },
        {
            'name': "ball_hits_bouncer",
            'type': "Contact",
            'args': {
                'first': "ball1",
                'second': "ball1_bouncer"
            },
            'children': [
                "ball_hits_top_lever",
            ]
        },
        {
            'name': "ball_hits_top_lever",
            'type': "Contact",
            'args': {
                'first': "ball1",
                'second': "lever1"
            },
            'children': [
                "plank_topples",
            ]
        },
        {
            'name': "plank_topples",
            'type': "Toppling",
            'args': {
                'body': "falling_plank",
                'angle': PLANK_TOPPLING_ANGLE
            },
            'children': [
                "plank_enters_goblet",
            ]
        },
        {
            'name': "plank_enters_goblet",
            'type': "Inclusion",
            'args': {
                'inside': "falling_plank",
                'outside': "lever3_goblet",
            },
            'children': [
                "bottom_lever_pivots",
            ]
        },
        {
            'name': "bottom_lever_pivots",
            'type': "Pivoting",
            'args': {
                'body': "lever3_goblet",
                'min_angvel': PIVOTING_ANGULAR_VELOCITY,
            },
            'children': [
                "bottom_mass_falls",
            ]
        },
        {
            'name': "bottom_mass_falls",
            'type': "Falling",
            'args': {
                'body': "lever4_weight",
                'min_linvel': FALLING_LINEAR_VELOCITY,
            },
            'children': [
                "coin_lever_pivots",
            ]
        },
        {
            'name': "coin_lever_pivots",
            'type': "Pivoting",
            'args': {
                'body': "coin_goblet_lever_plank",
                'min_angvel': PIVOTING_ANGULAR_VELOCITY,
            },
            'children': [
                "coin_falls",
            ]
        },
        {
            'name': "coin_falls",
            'type': "Falling",
            'args': {
                'body': "coin3",
                'min_linvel': FALLING_LINEAR_VELOCITY,
            },
            'children': [
                "coin_enters_box",
            ]
        },
        {
            'name': "coin_enters_box",
            'type': "Inclusion",
            'args': {
                'inside': "coin4",
                'outside': "box",
            },
        },
    ]
}
